package com.ruoyi.web;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.domain.ReviewKeyPoint;
import com.ruoyi.domain.ReviewSceneChangeTask;
import com.ruoyi.mapper.ReviewUsecaseMapper;
import com.ruoyi.service.ReviewDetailsService;
import com.ruoyi.service.ReviewSceneChangeTaskService;
import com.ruoyi.service.ReviewStandardService;
import com.ruoyi.service.ReviewUsecaseService;
import com.ruoyi.web.request.ReviewSceneChangeTaskCreateRequest;
import com.ruoyi.web.request.ReviewSceneChangeTaskPassRequest;
import com.ruoyi.web.response.*;
import com.ruoyi.web.request.*;
import com.ruoyi.web.response.ReviewSceneChangeTaskResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.skywalking.apm.toolkit.trace.Tag;
import org.apache.skywalking.apm.toolkit.trace.Tags;
import org.apache.skywalking.apm.toolkit.trace.Trace;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Api(tags = "场景变更任务")
@RestController
@RequestMapping("/review/scene/change/task")
public class ReviewSceneChangeTaskController extends BaseController {

    @Autowired
    private ReviewSceneChangeTaskService reviewSceneChangeTaskService;

    @Autowired
    private ReviewUsecaseService reviewUsecaseService;

    @Autowired
    private ReviewStandardService reviewStandardService;

    @Autowired
    private ReviewDetailsService reviewDetailsService;

    @ApiOperation("新建场景变更任务")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "场景变更任务", businessType = BusinessType.INSERT)
    @RequestMapping(method = RequestMethod.POST, value = "/add")
    public R<String> create(@Validated @RequestBody ReviewSceneChangeTaskCreateRequest request) {

        ReviewSceneChangeTask reviewSceneChangeTask = new ReviewSceneChangeTask();
        BeanUtils.copyBeanProp(reviewSceneChangeTask, request);
        /*reviewSceneChangeTask.setOperationSort(request.getOperation()+"_"+request.getSort());*/
        reviewSceneChangeTaskService.create(reviewSceneChangeTask);

        return R.ok();
    }

    @ApiOperation("审核任务")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "审核任务", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/audit")
    public R<String> audit(@Validated @RequestBody ReviewSceneChangeTaskPassRequest request) {
        reviewSceneChangeTaskService.audit(request);
        return R.ok();
    }

    /**
     * 根据用户ID查询待办列表
     * @return
     */
    @ApiOperation("根据用户ID查询待办列表")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/findPending")
    public TableDataInfo<ReviewSceneChangeTaskResponse> findPending(@Validated @RequestBody ReviewSceneChangeTaskFindPendingRequest request) {
        startPage(request);
        return getDataTable(reviewSceneChangeTaskService.findPendingListByUserId(getUserId(),request));
    }

    /**
     * 根据用户ID查询已办列表
     * @return
     */
    @ApiOperation("根据用户ID查询已办列表")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/findFinish")
    public TableDataInfo<ReviewSceneChangeTaskResponse> findFinish(@Validated @RequestBody ReviewSceneChangeTaskFindFinishRequest request) {
        startPage(request);
        return getDataTable(reviewSceneChangeTaskService.findFinishListByUserId(getUserId(),request));
    }

    /**
     * 查询当前用户发布的任务
     * @return
     */
    /*@ApiOperation("查询当前用户发布的任务")*/
    @ApiOperation("用户-评审管理-我发起的评审")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/findCreate")
    public TableDataInfo<ReviewSceneChangeTask> findCreate(@Validated @RequestBody ReviewSceneChangeTaskFindCreateRequest request) {
        startPage(request);
        return getDataTable(reviewSceneChangeTaskService.findCreateListByUserId(getUserId(),request));
    }

    @ApiOperation("查询任务详情")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/getById")
    public R<ReviewSceneChangeTask> getInfo(@Validated @RequestBody ReviewSceneChangeTaskGetInfoRequest request) {
        Long id = Long.parseLong(request.getId());
        return R.ok(reviewSceneChangeTaskService.getBySceneChangeTaskId(id));
    }

    /**
     * 创新基地查询待办列表
     * @return
     */
    @ApiOperation("创新基地查询待办列表")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/innovationBaseFindPending")
    public TableDataInfo<ReviewSceneChangeTaskResponse> innovationBaseFindPending(@Validated @RequestBody ReviewSceneChangeTaskFindPendingRequest request) {
        startPage(request);
        return getDataTable(reviewSceneChangeTaskService.findPendingList(getUserId(),request));
    }

    /**
     * 创新基地查询已办列表
     * @return
     */
    @ApiOperation("创新基地查询已办列表")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/innovationBaseFindFinish")
    public TableDataInfo<ReviewSceneChangeTaskResponse> innovationBaseFindFinish(@Validated @RequestBody ReviewSceneChangeTaskFindFinishRequest request) {
        startPage(request);
        return getDataTable(reviewSceneChangeTaskService.findFinishList(getUserId(),request));
    }

    /**
     * 创新基地查询任务详情
     * @return
     */
    @ApiOperation("查询任务详情")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/innovationBaseGetById")
    public R<ReviewSceneChangeTask> innovationBaseGetById(@Validated @RequestBody ReviewSceneChangeTaskGetInfoRequest request) {
        Long id = Long.parseLong(request.getId());
        return R.ok(reviewSceneChangeTaskService.getBySceneChange(id));
    }

    /**
     * 创新基地审核任务
     * @return
     */
    @ApiOperation("审核任务")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "审核任务", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/innovationBaseAudit")
    public R<String> innovationBaseAudit(@Validated @RequestBody ReviewSceneChangeTaskPassRequest request) {
        reviewSceneChangeTaskService.innovationBaseAudit(request);
        return R.ok();
    }

    /*
    * 发起评审-获取要点列表
    * */
    @ApiOperation("发起评审-获取要点列表")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "发起评审-获取要点列表", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/getKeyPointList")
    public R<List<ReviewKeyPointListResponse>> getKeyPointList(@Validated @RequestBody ReviewKeyPointListRequest request){
        Long id = Long.parseLong(request.getStandardId());
        List<ReviewKeyPointListResponse> list = reviewSceneChangeTaskService.getKeyPointList(id);
        return R.ok(list);
    }

    /*
     * 发起评审-获取细则列表
     * */
    @ApiOperation("发起评审-获取细则列表")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "发起评审-获取细则列表", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/getDetailsList")
    public R<List<ReviewDetailsListResponse>> getKeyPointList(@Validated @RequestBody ReviewDetailsListRequest request){
        Long id = Long.parseLong(request.getKeypointId());
        List<ReviewDetailsListResponse> list = reviewSceneChangeTaskService.getDetailsList(id);
        return R.ok(list);
    }

    /*
    * 发起评审-获取场景列表
    * */
    @ApiOperation("发起评审-获取场景列表")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "发起评审-获取场景列表", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/getSceneList")
    public R<List<ReviewSceneListResponse>> getSceneList(@Validated @RequestBody ReviewSceneListRequest request){
        Long id = Long.parseLong(request.getDetailsId());
        List<ReviewSceneListResponse> list = reviewSceneChangeTaskService.getSceneList(id);
        return R.ok(list);
    }

    /*
    * 获取评审人员
    * */
    @ApiOperation("获取评审人员")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "获取评审人员", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/getReviewerList")
    public R<List<SysUser>> getReviewerList(){
        List<SysUser> list = reviewSceneChangeTaskService.getReviewList();
        return R.ok(list);
    }

    /*
    * 查看评审人员评审意见
    * */
    @ApiOperation("查看评审人员评审意见")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "查看评审人员评审意见", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/getReviewerOpinion")
    public R<List<ReviewOpinionResponse>> getReviewerOpinion(@Validated @RequestBody ReviewSceneChangeTaskGetInfoRequest request){
        Long id = Long.parseLong(request.getId());
        return R.ok(reviewSceneChangeTaskService.getReviewerOpinion(id));
    }

    /*
    * 查询测试用例
    * */
    @ApiOperation("查询所有测试用例")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "查询所有测试用例", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/getUseCaseListAll")
    public R<List<GetUseCaseResponse>> getUseCaseListAll(@RequestBody GetUseCaseRequestNew GetUseCaseRequestNew){
        List<Long> idList = reviewStandardService.getIdList(GetUseCaseRequestNew);
        List<GetUseCaseResponse> getUseCaseResponseList = reviewUsecaseService.getUseCaseListByIds(idList);
        return R.ok(getUseCaseResponseList);
    }

    /*
    * 新增细则按钮
    * */
    @ApiOperation("新增细则按钮")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "新增细则按钮", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/getKeypointListAll")
    public R<List<AddDetailsResponse>> getDetailsListNew(@RequestBody GetKeypointListRequest getKeypointListRequest){
        List<Long> idList = reviewStandardService.getKeypointIdList(getKeypointListRequest);
        List<AddDetailsResponse> addDetailsResponseList = reviewStandardService.getKeypointListByIds(idList);
        return R.ok(addDetailsResponseList);
    }

    /*
     * 删除细则按钮
     * */
    @ApiOperation("删除细则按钮")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "删除细则按钮", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/deleteKeypointListAll")
    public R<List<DeleteDetailsResponse>> deleteKeypointListAll(@RequestBody DeleteDetailsRequest deleteDetailsRequest){
        List<DeleteDetailsResponse> responseList = reviewDetailsService.getDeleteDetailsResponseList(deleteDetailsRequest);
        return R.ok(responseList);
    }

}
